@@ -7,11 +7,15 @@
   bill@newbreedsoftware.com
   http://www.newbreedsoftware.com/
   
-  March 19, 2000 - November 12, 2001
+  March 19, 2000 - April 20, 2002
 */
 
 
-#define VERSION "0.0.4"
+#ifndef EMBEDDED
+#define VERSION "0.0.6"
+#else
+#define VERSION "0.0.6-embedded"
+#endif
 
 
 /* #ncludes: */
@@ -28,8 +32,18 @@
 #include <SDL_mixer.h>
 #endif
 
+#ifndef EMBEDDED
 #include "data/images/flyswatter.xbm"
 #include "data/images/flyswatter-mask.xbm"
+#endif
+
+
+#ifdef LINUX
+#define LINUX_DAT
+#endif
+#ifdef EMBEDDED
+#define LINUX_DAT
+#endif
 
 
 #define POINTS_FOR_BONUS_BLOOD 10000 /* How many points til bonus blood */
@@ -40,6 +54,15 @@
 #define CHANCE_OF_SPORATICNESS 20    /* How often to change direction */
 
 
+#ifndef EMBEDDED
+#define WIDTH 640
+#define HEIGHT 480
+#else
+#define WIDTH 240
+#define HEIGHT 320
+#endif
+
+
 /* Image enumerations: */
 
 enum {
@@ -88,6 +111,9 @@ enum {
   IMG_MULTIPLIER,
   IMG_MULTIPLIER_SQUISHED,
   IMG_TIMES2,
+#ifdef EMBEDDED
+  IMG_START_CONT,
+#endif
   NUM_IMAGES
 };
 
@@ -140,6 +166,10 @@ const char * image_names [NUM_IMAGES] = {
   DATA_PREFIX "images/mult.png",
   DATA_PREFIX "images/mult-squished.png",
   DATA_PREFIX "images/times2.png"
+#ifdef EMBEDDED
+  ,
+  DATA_PREFIX "images/start_cont.png"
+#endif
 };
 
 
@@ -198,6 +228,9 @@ typedef struct bug_type {
 
 typedef struct splat_type {
   int active, img, x, y;
+#ifdef EMBEDDED
+  int timer;
+#endif
 } splat_type;
 
 typedef struct extra_type {
@@ -230,6 +263,11 @@ void savedata(void);
 FILE * opendata(char * mode);
 void getargs(int argc, char * argv[]);
 void usage(int ret);
+#ifdef EMBEDDED
+FILE * openstate(char * mode);
+void loadstate(void);
+void savestate(void);
+#endif
 
 /* Global variables: */
 
@@ -238,6 +276,10 @@ SDL_Surface * images[NUM_IMAGES];
 int use_fullscreen, use_sound;
 int level, score, highscore, highlevel, blood, bugs_added, multiplier,
   has_highscore;
+int num_bugs, next_level, old_blood, bonus;
+#ifdef EMBEDDED
+int can_continue;
+#endif
 bug_type bugs[MAX_BUGS];
 splat_type splats[MAX_SPLATS];
 extra_type extra;
@@ -268,6 +310,14 @@ int main(int argc, char * argv[])
   /* Load data: */
   
   loaddata();
+
+
+#ifdef EMBEDDED
+
+  /* Load game state: */
+
+  loadstate();
+#endif
   
   
   /* Stress warning: */
@@ -303,7 +353,15 @@ int main(int argc, char * argv[])
   
   savedata();
   
-  
+
+#ifdef EMBEDDED
+
+  /* Save game state: */
+
+  savestate();
+#endif
+ 
+
   /* Quit and exit: */
   
   SDL_Quit();
@@ -318,12 +376,12 @@ int game(void)
 {
   SDL_Rect src, dest;
   SDL_Event event;
-  int done, quit, gameover, i, frame, num_bugs, next_level, old_blood, bonus,
-    mouse_x, mouse_y;
+  int done, quit, gameover, i, frame, mouse_x, mouse_y, j;
   
   
   /* Start the game! */
-  
+ 
+#ifndef EMBEDDED
   level = 1;
   score = 0;
   has_highscore = 0;
@@ -332,11 +390,29 @@ int game(void)
   old_blood = 0;
   bonus = 0;
   next_level = 0;
+
+  resetlevel();
+#else
+  if (can_continue == 0)
+  {
+    level = 1;
+    score = 0;
+    has_highscore = 0;
+    multiplier = 1;
+    blood = (images[IMG_GAUGE_FULL] -> h);
+    old_blood = 0;
+    bonus = 0;
+    next_level = 0;
+    
+    resetlevel();
+  }
+
+  can_continue = 1;
+#endif
+  
   mouse_x = 0;
   mouse_y = 0;
   
-  resetlevel();
-  
   
   /* MAIN GAME LOOP! */
   
@@ -372,6 +448,7 @@ int game(void)
 		  quit = pause_screen();
 		}
 	    }
+#ifndef EMBEDDED
           else if (event.type == SDL_MOUSEMOTION)
             {
               mouse_x = event.motion.x;
@@ -379,6 +456,15 @@ int game(void)
             }
 	  else if (event.type == SDL_MOUSEBUTTONDOWN)
 	    {
+#else
+          else if (event.type == SDL_MOUSEMOTION)
+            {
+              mouse_x = event.motion.x;
+              mouse_y = event.motion.y;
+
+	      event.button.x = mouse_x;
+	      event.button.y = mouse_y;
+#endif
 	      if (!gameover)
 		{
 		  /* Mouse click! - Squish! */
@@ -395,7 +481,7 @@ int game(void)
 			  playsound(SND_SQUISH1 + (rand() % 2), 0);
 			  addsplat(bugs[i].x, bugs[i].y,
 				   (IMG_BUG0C + bugs[i].kind * 3));
-			  incrementscore((480 - bugs[i].y) / 4);
+			  incrementscore((HEIGHT - bugs[i].y) / 4);
 			}
 		    }
 		  
@@ -449,7 +535,15 @@ int game(void)
 	      /* Move: */
 	      
 	      bugs[i].x = bugs[i].x + bugs[i].xm;
+
+#ifndef EMBEDDED
 	      bugs[i].y = bugs[i].y + bugs[i].ym;
+#else
+	      if (bugs[i].ym == 1 && (frame % 2) == 0)
+		bugs[i].y++;
+	      else
+		bugs[i].y = bugs[i].y + (bugs[i].ym / 2);
+#endif
 	      
 	      
 	      if (bugs[i].kind == BUG_MOSQUITO)
@@ -518,9 +612,9 @@ int game(void)
 		  bugs[i].x = images[IMG_GAUGE_FULL] -> w;
 		  bugs[i].xm = 1;
 		}
-	      else if (bugs[i].x > 607)
+	      else if (bugs[i].x > WIDTH - 33)
 		{
-		  bugs[i].x = 607;
+		  bugs[i].x = WIDTH - 33;
 		  bugs[i].xm = -1;
 		}
 	      
@@ -551,6 +645,9 @@ int game(void)
 			  if (!gameover)
 			    {
 			      gameover = 1;
+#ifdef EMBEDDED
+			      can_continue = 0;
+#endif
 			      playsound(SND_UGH, 2);
 			      
 			      
@@ -578,8 +675,8 @@ int game(void)
 	  
 	  /* Out of bounds? */
 	  
-	  if (extra.x < -32 || extra.x > 640 ||
-	      extra.y < -32 || extra.y > 480)
+	  if (extra.x < -32 || extra.x > WIDTH ||
+	      extra.y < -32 || extra.y > HEIGHT)
 	    {
 	      extra.active = 0;
 	    }
@@ -616,7 +713,7 @@ int game(void)
 		{
 		  /* Left: */
 		  
-		  extra.x = 640;
+		  extra.x = WIDTH;
 		  extra.xm = -(rand() % 16);
 		}
 	      
@@ -639,8 +736,12 @@ int game(void)
       /* Draw arm: */
       
       dest.x = 0;
-      dest.y = 480 - images[IMG_ARM1] -> h;
-      dest.w = 640;
+#ifndef EMBEDDED
+      dest.y = HEIGHT - images[IMG_ARM1] -> h;
+#else
+      dest.y = HEIGHT - images[IMG_ARM1] -> h - 40;
+#endif
+      dest.w = WIDTH;
       dest.h = images[IMG_ARM1] -> h;
       
       if (!next_level)
@@ -673,9 +774,40 @@ int game(void)
 	      dest.y = splats[i].y;
 	      dest.w = 32;
 	      dest.h = 32;
-	      
+	    
+#ifndef EMBEDDED
 	      SDL_BlitSurface(images[splats[i].img], NULL,
-			      screen, &dest);
+		              screen, &dest);
+#else
+	      if (splats[i].timer > 10)
+	      {
+	        SDL_BlitSurface(images[splats[i].img], NULL,
+			        screen, &dest);
+	      }
+	      else
+	      {
+		for (j = (splats[i].timer % 2); j < 32; j = j + 2)
+		{
+	          dest.x = splats[i].x;
+		  dest.y = splats[i].y + j;
+		  dest.w = 32;
+	          dest.h = 1;
+
+		  src.x = 0;
+		  src.y = j;
+		  src.w = 32;
+		  src.h = 1;
+
+		  SDL_BlitSurface(images[splats[i].img], &src,
+				  screen, &dest);
+		}
+	      }
+
+              splats[i].timer--;
+		
+	      if (splats[i].timer == 0)
+	        splats[i].active = 0;
+#endif
 	    }
 	}
       
@@ -771,13 +903,13 @@ int game(void)
       /* Score: */
       
       dest.x = 0;
-      dest.y = 440;
+      dest.y = HEIGHT - 40;
       dest.w = 128;
       dest.h = 40;
       
       SDL_BlitSurface(images[IMG_SCOREBOX], NULL, screen, &dest);
       
-      drawdigits(score, 2, 462);
+      drawdigits(score, 2, HEIGHT - 18);
       
       
       /* High score bubble? */
@@ -785,7 +917,7 @@ int game(void)
       if (has_highscore)
 	{
 	  dest.x = 110;
-	  dest.y = 462;
+	  dest.y = HEIGHT - 18;
 	  dest.w = 16;
 	  dest.h = 16;
 	  
@@ -795,14 +927,14 @@ int game(void)
 
       /* Wave: */
       
-      dest.x = 576;
-      dest.y = 440;
+      dest.x = WIDTH - 64;
+      dest.y = HEIGHT - 40;
       dest.w = 64;
       dest.h = 40;
       
       SDL_BlitSurface(images[IMG_WAVEBOX], NULL, screen, &dest);
       
-      drawdigits(level, 578, 462);
+      drawdigits(level, WIDTH - 62, HEIGHT - 18);
       
       
       /* Go to next level? */
@@ -856,10 +988,8 @@ int game(void)
 	  
 	  /* Draw bonus box: */
 	  
-	  dest.x = 192;
-	  dest.y = 176;
-	  dest.w = 256;
-	  dest.h = 128;
+	  dest.x = (WIDTH - images[IMG_BONUS]->w) / 2;
+	  dest.y = (HEIGHT - images[IMG_BONUS]->h) / 2;
 	  
 	  SDL_BlitSurface(images[IMG_BONUS], NULL,
 			  screen, &dest);
@@ -969,28 +1099,29 @@ void setup(void)
   
   if (use_fullscreen == 1)
     {
-      screen = SDL_SetVideoMode(640, 480, 16, SDL_FULLSCREEN | SDL_HWSURFACE);
+      screen = SDL_SetVideoMode(WIDTH, HEIGHT, 16,
+		                SDL_FULLSCREEN | SDL_HWSURFACE);
       if (screen == NULL)
         {
           fprintf(stderr,
                   "\nWarning: I could not set up fullscreen video for "
-                  "640x480 mode.\n"
+                  "%d x %d mode.\n"
                   "The Simple DirectMedia error that occured was:\n"
-                  "%s\n\n", SDL_GetError());
+                  "%s\n\n", WIDTH, HEIGHT, SDL_GetError());
           use_fullscreen = 0;
         }
     }
   
   if (use_fullscreen == 0)
     {
-      screen = SDL_SetVideoMode(640, 480, 16, 0);
+      screen = SDL_SetVideoMode(WIDTH, HEIGHT, 16, 0);
       
       if (screen == NULL)
         {
           fprintf(stderr,
-                  "\nError: I could not set up video for 640x480 mode.\n"
+                  "\nError: I could not set up video for %d x %d mode.\n"
                   "The Simple DirectMedia error that occured was:\n"
-                  "%s\n\n", SDL_GetError());
+                  "%s\n\n", WIDTH, HEIGHT, SDL_GetError());
           exit(1);
         }
     }
@@ -1006,6 +1137,7 @@ void setup(void)
   SDL_WM_SetCaption("Bug Squish", "Bug Squish");
 
 
+#ifndef EMBEDDED
   /* Make cursor: */
   
   for (i = 0; i < 32; i++)
@@ -1034,6 +1166,7 @@ void setup(void)
     }
   
   SDL_SetCursor(SDL_CreateCursor(temp_bitmap, temp_bitmask, 16, 16, 0, 0));
+#endif
   
   
   /* Load graphics: */
@@ -1041,8 +1174,12 @@ void setup(void)
   for (i = 0; i < NUM_IMAGES; i++)
     {
       /* Load image file: */
-      
+     
+#ifndef EMBEDDED
       image = IMG_Load(image_names[i]);
+#else
+      image = SDL_LoadBMP(image_names[i]);
+#endif
 
       if (image == NULL)
         {
@@ -1056,8 +1193,8 @@ void setup(void)
       
       
       /* Set transparency: */
-      
-      /*
+     
+#ifdef EMBEDDED
       images[i] = SDL_DisplayFormat(image);
       if (images[i] == NULL)
         {
@@ -1082,22 +1219,33 @@ void setup(void)
         }
 
       SDL_FreeSurface(image);
-      */
 
+#else
       images[i] = image;
+#endif
       
       
       /* Draw percentage bar: */
       
       dest.x = 0;
-      dest.y = 470;
-      dest.w = (640 * i) / NUM_IMAGES;
+      dest.y = HEIGHT - 10;
+      dest.w = (WIDTH * i) / NUM_IMAGES;
       dest.h = 10;
       
       SDL_FillRect(screen, &dest, SDL_MapRGB(screen->format, 255, 255, 255));
       SDL_UpdateRect(screen, dest.x, dest.y, dest.w, dest.h);
       SDL_Delay(1);
     }
+
+
+#ifndef NOSOUND
+#ifdef EMBEDDED
+  if (use_sound == 1)
+  {
+    Mix_Volume(-1, MIX_MAX_VOLUME);
+  }
+#endif
+#endif
   
   
 #ifndef NOSOUND
@@ -1164,7 +1312,7 @@ int title(void)
 {
   int done, quit, i, x, y;
   SDL_Event event;
-  SDL_Rect dest;
+  SDL_Rect dest, src;
 
   
   /* Start music, if it's not playing: */
@@ -1179,14 +1327,18 @@ int title(void)
     }
 #endif
   
-  
+ 
   /* Animate introduction of title screen: */
-  
-  for (i = 0; i <= 320; i = i + 32)
+ 
+#ifndef EMBEDDED
+  for (i = 0; i <= WIDTH / 2; i = i + 32)
+#else
+  for (i = 0; i <= HEIGHT / 2; i = i + 32)
+#endif
     {
-      for (y = 240 - i; y < 240 + i; y = y + 16)
+      for (y = HEIGHT / 2 - i; y < HEIGHT / 2 + i; y = y + 16)
 	{
-	  for (x = 320 - i; x < 320 + i; x = x + 16)
+	  for (x = WIDTH / 2 - i; x < WIDTH / 2 + i; x = x + 16)
 	    {
 	      dest.x = x;
 	      dest.y = y;
@@ -1205,14 +1357,38 @@ int title(void)
   
   /* Draw score: */
   
-  drawdigits(score, 466, 422);
-  drawdigits(level, 600, 422);
+  drawdigits(score, WIDTH - 174, HEIGHT - 58);
+  drawdigits(level, WIDTH - 40, HEIGHT - 58);
   
   
   /* Draw high score: */
   
-  drawdigits(highscore, 466, 460);
-  drawdigits(highlevel, 600, 460);
+  drawdigits(highscore, WIDTH - 174, HEIGHT - 21);
+  drawdigits(highlevel, WIDTH - 40, HEIGHT - 21);
+
+
+#ifdef EMBEDDED
+  /* Draw start & continue buttons: */
+
+  dest.x = 54;
+  dest.y = 158;
+
+  if (can_continue)
+  {
+    SDL_BlitSurface(images[IMG_START_CONT], NULL,
+		    screen, &dest);
+  }
+  else
+  {
+    src.x = 0;
+    src.y = 0;
+    src.w = images[IMG_START_CONT]->w;
+    src.h = 40;
+
+    SDL_BlitSurface(images[IMG_START_CONT], &src,
+		    screen, &dest);
+  }
+#endif
   
 
   SDL_Flip(screen);
@@ -1246,8 +1422,27 @@ int title(void)
 	  else if (event.type == SDL_MOUSEBUTTONDOWN)
 	    {
 	      /* Mouse click! - Start! */
-	      
+
+#ifndef EMBEDDED
 	      done = 1;
+#else
+	      if (event.button.x >= 54 &&
+	          event.button.x < 54 + images[IMG_START_CONT]->w)
+	      {
+		if (event.button.y >= 158 &&
+		    event.button.y < 158 + images[IMG_START_CONT]->h / 2)
+		{
+	          can_continue = 0;
+		  done = 1;
+		}
+		else if (event.button.y >= 158 + images[IMG_START_CONT]->h / 2
+			 && event.button.y < 158 + images[IMG_START_CONT]->h &&
+			 can_continue)
+		{
+		  done = 1;
+		}
+	      }
+#endif
 	    }
 	}
       
@@ -1288,6 +1483,7 @@ int title(void)
 
 void seticon(void)
 {
+#ifndef EMBEDDED
   int masklen;
   Uint8 * mask;
   SDL_Surface * icon;
@@ -1322,6 +1518,7 @@ void seticon(void)
   
   free(mask);
   SDL_FreeSurface(icon);
+#endif
 }
 
 
@@ -1336,10 +1533,8 @@ int pause_screen(void)
   
   /* Draw "Paused" notice: */
   
-  dest.x = 192;
-  dest.y = 208;
-  dest.w = 256;
-  dest.h = 64;
+  dest.x = (WIDTH - images[IMG_PAUSED]->w) / 2;
+  dest.y = (HEIGHT - images[IMG_PAUSED]->h) / 2;
   
   SDL_BlitSurface(images[IMG_PAUSED], NULL, screen, &dest);
   SDL_UpdateRect(screen, dest.x, dest.y, dest.w, dest.h);
@@ -1401,6 +1596,9 @@ int pause_screen(void)
 void playsound(int snd, int chan)
 {
 #ifndef NOSOUND
+#ifndef EMBEDDED
+  chan = -1;
+#endif
   if (use_sound)
     Mix_PlayChannel(chan, sounds[snd], 0);
 #endif
@@ -1467,7 +1665,7 @@ void addbug(void)
       bugs[found].alive = 1;
       
       bugs[found].kind = ((rand() % ((level / 3) + 1)) % NUM_BUGS);
-      bugs[found].x = (rand() % 608);
+      bugs[found].x = (rand() % WIDTH - 32);
       bugs[found].y = -30;
       bugs[found].xm = (rand() % (level * 2)) - level;
       bugs[found].xmm = (rand() % 5) - 2;
@@ -1479,8 +1677,11 @@ void addbug(void)
       if (bugs[found].ym > 24)
 	bugs[found].ym = 24;
       
-      bugs[found].target_y = ((480 - images[IMG_ARM1] -> h) +
+      bugs[found].target_y = ((HEIGHT - images[IMG_ARM1] -> h) +
 			      (rand() % ((images[IMG_ARM1] -> h) / 2)));
+#ifdef EMBEDDED
+      bugs[found].target_y -= 40;
+#endif
       
       bugs_added++;
     }
@@ -1513,6 +1714,9 @@ void addsplat(int x, int y, int img)
       splats[found].img = img;
       splats[found].x = x;
       splats[found].y = y;
+#ifdef EMBEDDED
+      splats[found].timer = 50;
+#endif
     }
 }
 
@@ -1544,7 +1748,8 @@ void intro(void)
   
   scalespeed = 0.1;
   done = 0;
-  
+ 
+#ifndef EMBEDDED
   for (scale = 1; scale < 5 && done == 0; scale = scale + scalespeed)
     {
       scalespeed = scalespeed + 0.1;
@@ -1573,8 +1778,8 @@ void intro(void)
 	      
 	      /* Duplicate it as a rectangle: */
 	      
-	      dest.x = 320 - (64 * scale) + x * scale;
-	      dest.y = 240 - (48 * scale) + y * scale;
+	      dest.x = WIDTH / 2 - (64 * scale) + x * scale;
+	      dest.y = HEIGHT / 2 - (48 * scale) + y * scale;
 	      dest.w = scale + 1;
 	      dest.h = scale + 1;
 	      
@@ -1596,6 +1801,13 @@ void intro(void)
 	    done = 1;
 	}
     }
+#else
+  dest.x = (WIDTH - images[IMG_PRESENTS]->w) / 2;
+  dest.y = (HEIGHT - images[IMG_PRESENTS]->h) / 2;
+  SDL_BlitSurface(images[IMG_PRESENTS], NULL, screen, &dest);
+  SDL_Flip(screen);
+  SDL_Delay(2000);
+#endif
   
   
   /* Pause for a moment before erasing intro. screen... */
@@ -1632,7 +1844,11 @@ void drawbonusnumbers(int num)
   
   place = 10000;
   nonzero = 0;
+#ifndef EMBEDDED
   x = 286;
+#else
+  x = (WIDTH / 2) - images[IMG_LARGE0]->w;
+#endif
   
   for (i = 0; i < 5; i++)
     {
@@ -1644,14 +1860,12 @@ void drawbonusnumbers(int num)
       if (digit != 0 || nonzero)
 	{
 	  dest.x = x;
-	  dest.y = 240;
-	  dest.w = images[IMG_LARGE0 + digit] -> w;
-	  dest.h = images[IMG_LARGE0 + digit] -> h;
+	  dest.y = HEIGHT / 2;
 	  
 	  SDL_BlitSurface(images[IMG_LARGE0 + digit], NULL,
 			  screen, &dest);
 	  
-	  x = x + dest.w;
+	  x = x + images[IMG_LARGE0 + digit] -> w;
 	}
       
       num = num - (digit * place);
@@ -1792,6 +2006,11 @@ void loaddata(void)
       
       fclose(fi);
     }
+
+
+#ifdef EMBEDDED
+  can_continue = 0;
+#endif
 }
 
 
@@ -1822,15 +2041,16 @@ void savedata(void)
 }
 
 
-/* Open the data file: */
+#ifdef EMBEDDED
 
-FILE * opendata(char * mode)
+/* Open the state file: */
+
+FILE * openstate(char * mode)
 {
   char * filename, * home;
   FILE * fi;
   
   
-#ifdef LINUX
   /* Get home directory (from $HOME variable)... if we can't determine it,
      use the current directory ("."): */
   
@@ -1843,7 +2063,59 @@ FILE * opendata(char * mode)
   /* Create the buffer for the filename: */
   
   filename = (char *) malloc(sizeof(char) * (strlen(home) +
-                                             strlen("/.madbomber") + 1));
+                                             strlen("/.bugsquish-state") + 1));
+  
+  strcpy(filename, home);
+  strcat(filename, "/.bugsquish-state");
+  
+  
+  /* Try opening the file: */
+  
+  fi = fopen(filename, mode);
+  
+  if (fi == NULL)
+    {
+      fprintf(stderr, "\nWarning: I could not open the state file "); 
+      
+      if (strcmp(mode, "r") == 0)
+        fprintf(stderr, "for read:");
+      else if (strcmp(mode, "w") == 0)
+        fprintf(stderr, "for write:");
+      
+     fprintf(stderr, "\n%s\n"
+              "The error that occured was:\n"
+              "%s\n\n", filename, strerror(errno));
+    }
+  
+  
+  return(fi);
+}
+
+#endif
+
+
+/* Open the data file: */
+
+FILE * opendata(char * mode)
+{
+  char * filename, * home;
+  FILE * fi;
+  
+  
+#ifdef LINUX_DAT
+  /* Get home directory (from $HOME variable)... if we can't determine it,
+     use the current directory ("."): */
+  
+  if (getenv("HOME") != NULL)
+    home = getenv("HOME");
+  else
+    home = ".";
+  
+  
+  /* Create the buffer for the filename: */
+  
+  filename = (char *) malloc(sizeof(char) * (strlen(home) +
+                                             strlen("/.bugsquish") + 1));
   
   strcpy(filename, home);
   strcat(filename, "/.bugsquish");
@@ -1956,3 +2228,73 @@ void usage(int ret)
   
   exit(ret);
 }
+
+
+#ifdef EMBEDDED
+
+void loadstate(void)
+{
+  FILE * fi;
+
+  fi = openstate("r");
+  if (fi != NULL)
+  {
+    can_continue = fgetc(fi);
+
+    if (can_continue)
+    {
+      level = fgetc(fi);
+      fread(&score, sizeof(int), 1, fi);
+      fread(&blood, sizeof(int), 1, fi);
+      fread(&bugs_added, sizeof(int), 1, fi);
+      multiplier = fgetc(fi);
+      has_highscore = fgetc(fi);
+      fread(&num_bugs, sizeof(int), 1, fi);
+      next_level = fgetc(fi);
+      old_blood = fgetc(fi);
+      fread(&bonus, sizeof(int), 1, fi);
+      fread(bugs, sizeof(bug_type), MAX_BUGS, fi);
+      fread(splats, sizeof(splat_type), MAX_SPLATS, fi);
+      fread(&extra, sizeof(extra_type), 1, fi);
+    }
+
+    fclose(fi);
+  }
+  else
+  {
+    can_continue = 0;
+  }
+}
+
+
+void savestate(void)
+{
+  FILE * fi;
+
+  fi = openstate("w");
+  if (fi != NULL)
+  {
+    fputc(can_continue, fi);
+
+    if (can_continue)
+    {
+      fputc(level, fi);
+      fwrite(&score, sizeof(int), 1, fi);
+      fwrite(&blood, sizeof(int), 1, fi);
+      fwrite(&bugs_added, sizeof(int), 1, fi);
+      fputc(multiplier, fi);
+      fputc(has_highscore, fi);
+      fwrite(&num_bugs, sizeof(int), 1, fi);
+      fputc(next_level, fi);
+      fputc(old_blood, fi);
+      fwrite(&bonus, sizeof(int), 1, fi);
+      fwrite(bugs, sizeof(bug_type), MAX_BUGS, fi);
+      fwrite(splats, sizeof(splat_type), MAX_SPLATS, fi);
+      fwrite(&extra, sizeof(extra_type), 1, fi);
+    }
+
+    fclose(fi);
+  }
+}
+
+#endif
